@@ -1,15 +1,15 @@
 ;;; double-press.el --- Double-press key dispatcher -*- lexical-binding: t; -*-
 ;; SPDX-License-Identifier: GPL-3.0-or-later
 
-;; Copyright (C) 2010-2012, 2025 K-talo Miyazaki, all rights reserved.
+;; Copyright (C) 2010-2012, 2025 K-talo Miyazaki
 
-;; Author: K-talo Miyazaki <Keitaro dot Miyazaki at gmail dot com>
+;; Author: K-talo Miyazaki <Keitaro.Miyazaki@gmail.com>
 ;; Created: Fri Dec 24 02:33:06 2010 JST
 ;; Keywords: abbrev, convenience, emulations, wp
 ;; URL: https://github.com/k-talo/double-press.el
-;; Maintainer: K-talo Miyazaki <Keitaro dot Miyazaki at gmail dot com>
-;; Package-Version: 3.0.1
-;; Package-Revision: v3.0.1-0-g35fc8b41f015
+;; Maintainer: K-talo Miyazaki <Keitaro.Miyazaki@gmail.com>
+;; Package-Version: 20250916.1005
+;; Package-Revision: 380f7f1cab90
 ;; Package-Requires: ((emacs "24.4"))
 
 ;; This file is not part of GNU Emacs.
@@ -44,8 +44,9 @@
 ;;     :on-double-press 'my-window-map)
 ;;
 ;; Customization:
-;;   - double-press-timeout    ;; maximum interval between presses (seconds)
-;;   - double-press-use-prompt ;; show a prompt when reading from a prefix map
+;;   - double-press-timeout              ;; maximum interval between presses (seconds)
+;;   - double-press-use-prompt           ;; show a prompt when reading from a prefix map
+;;   - double-press-use-where-is-helper  ;; keep `where-is' hints in sync (opt-in)
 ;;   - Press C-h/<f1> inside a double-press prefix to see its bindings.
 ;;
 ;; See also: README.md (overview, setup) and docs/EXAMPLES.md (snippets).
@@ -142,11 +143,11 @@ missed, raise it."
   "Bind KEY in KEYMAP to two actions: ON-SINGLE-PRESS and ON-DOUBLE-PRESS.
 
 Returns the dispatcher (an uninterned symbol bound to an interactive
-command) installed at KEY. The dispatcher decides between the two
+command) installed at KEY.  The dispatcher decides between the two
 bindings by timing: a single press or a quick double-press within
 `double-press-timeout\\=' seconds.
 
-KEY may be a string or a vector of events. Using [t] as KEY creates a
+KEY may be a string or a vector of events.  Using [t] as KEY creates a
 default definition used for any event not otherwise defined in KEYMAP.
 
 ON-SINGLE-PRESS and ON-DOUBLE-PRESS accept any key definition:
@@ -162,11 +163,12 @@ double-press bindings.
 
 Example:
 
-  (define-prefix-command \\='my-window-map)
-  (define-key my-window-map (kbd \"s\") \\='split-window-below)
-  (double-press-define-key global-map (kbd \"M-w\")
-    :on-single-press \\='copy-region-as-kill
-    :on-double-press \\='my-window-map)
+  ;; Escalate query replace: single press runs \\='query-replace; double press
+  ;; runs \\='query-replace-regexp.
+
+  (double-press-define-key global-map (kbd \"M-%\")
+                           :on-single-press \\='query-replace
+                           :on-double-press \\='query-replace-regexp)
 
 See also `define-key\\=', `double-press-timeout\\=', and
 `double-press-use-prompt\\='."
@@ -224,7 +226,46 @@ ignored."
         (and (keymapp single-map) (define-key single-map key nil))
         (and (keymapp double-map) (define-key double-map key nil))))))
 
-(advice-add 'define-key :before #'double-press--define-key-advice)
+;; Optional helper to clear [single]/[double] hints when keys are redefined.
+;; This is invasive (advises `define-key'), so keep it opt-in.
+;; Enable via Customize (e.g., `customize-set-variable' or `setopt' on Emacs 29.1+) or
+;; use `M-x double-press-activate-where-is-helper'.  Note: `setq' does not
+;; run the option's :set handler and will not toggle the advice.
+
+;;;###autoload
+(defun double-press-activate-where-is-helper ()
+  "Enable the `define-key' advice that clears [single]/[double] hints."
+  (interactive)
+  (unless (advice-member-p #'double-press--define-key-advice 'define-key)
+    (advice-add 'define-key :before #'double-press--define-key-advice)))
+
+;;;###autoload
+(defun double-press-deactivate-where-is-helper ()
+  "Disable the `define-key' advice that clears [single]/[double] hints."
+  (interactive)
+  (when (advice-member-p #'double-press--define-key-advice 'define-key)
+    (advice-remove 'define-key #'double-press--define-key-advice)))
+
+;;;###autoload
+(defcustom double-press-use-where-is-helper nil
+  "When non-nil, enable advice on `define-key' to clear `where-is' hints.
+
+This helper removes stale entries from the auxiliary [single]/[double]
+submaps so that `where-is' stays in sync when keys defined via
+`double-press-define-key' are later rebound.
+
+Because this advises a core function, it is disabled by default.  Enable
+it via Customize (e.g., `customize-set-variable' or `setopt' on Emacs 29.1+)
+or call `double-press-activate-where-is-helper'.  Disable via Customize or
+call `double-press-deactivate-where-is-helper'.  A plain `setq' does not run
+the option's :set handler and will not toggle the advice."
+  :type 'boolean
+  :group 'double-press
+  :set (lambda (sym val)
+         (set-default sym val)
+         (if val
+             (double-press-activate-where-is-helper)
+           (double-press-deactivate-where-is-helper))))
 
 
 
@@ -519,7 +560,7 @@ This helps `describe-key' show single-press and double-press bindings."
        (let* ((vec (vector ev))
               (key-desc (condition-case _
                             (key-description vec)
-                          (error (format "%S" ev))))
+                          (error "%S" ev)))
               (key-def-desc (double-press--doc-key-def-desc key-def)))
          (insert (format "  %s -> %s\n" key-desc key-def-desc))))
      keymap)
